"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Pool = _interopRequireDefault(require("mysql/lib/Pool"));

var _debug = _interopRequireDefault(require("debug"));

var _Connection = _interopRequireDefault(require("mysql/lib/Connection"));

const log = (0, _debug.default)("webiny-mysql-connection");

class MySQLConnection {
  constructor(instance) {
    // Will throw an error if an invalid instance was passed.
    this.constructor.validateMySQLInstance(instance); // If everything went okay, let's assign and continue.

    this.instance = instance;
  }

  getInstance() {
    return this.instance;
  }

  isConnectionPool() {
    return this.getInstance() instanceof _Pool.default;
  }

  isConnection() {
    return this.getInstance() instanceof _Connection.default;
  }

  static validateMySQLInstance(instance) {
    if (instance instanceof _Pool.default || instance instanceof _Connection.default) {
      return;
    }

    throw Error("A valid MySQL connection or pool must be passed.");
  }

  query(sql) {
    let results = [],
        queries = sql instanceof Array ? sql : [sql];
    log(sql);
    return new Promise(async (resolve, reject) => {
      if (this.isConnectionPool()) {
        return this.getInstance().getConnection(async (error, connection) => {
          if (error) {
            reject(error);
            return;
          }

          try {
            results = await this.__executeQueriesWithConnection(connection, queries);
          } catch (e) {
            connection.release();
            return reject(e);
          }

          connection.release();
          queries.length === 1 ? resolve(results[0]) : resolve(results);
        });
      } // We don't close the passed connection, because it might be used outside of the scope of entity.


      try {
        results = await this.__executeQueriesWithConnection(this.getInstance(), queries);
        queries.length === 1 ? resolve(results[0]) : resolve(results);
      } catch (e) {
        reject(e);
      }
    });
  }
  /**
   * Tests connection - just sends a simple query.
   */


  async test() {
    await this.query("SELECT version()");
    return true;
  }

  async __executeQueriesWithConnection(connection, queries) {
    const results = [];

    for (let i = 0; i < queries.length; i++) {
      results.push((await this.__executeQueryWithConnection(connection, queries[i])));
    }

    return results;
  }

  async __executeQueryWithConnection(connection, sql) {
    return new Promise((resolve, reject) => {
      connection.query(sql, (error, results) => {
        if (error) {
          return reject(error);
        }

        resolve(results);
      });
    });
  }

}

var _default = MySQLConnection;
exports.default = _default;
//# sourceMappingURL=mySQLConnection.js.map